%DEMO_HORNE2016 Demo of the phenomenological model by Horne et al. (2016)
%
%   This script generates a figure showing how the spiking probability and 
%   latency of an electrically stimulated auditory nerve fiber changes as a
%   function of stimulation level for a biphasic pulse, as predicted by the
%   phenomenological model by Horne et al. (2016).
%
%   Also other pulse types (monophasic, pseudo-monophasic, tri-phasic) can
%   be used as input to the model.
%
%   See also: horne2016.m
%
%   References:
%     C. Horne, C.J. Sumner, and B.U. Seeber. A phenomenological model of
%     the electrically stimulated auditory nerve fiber: temporal and
%     biphasic response properties. Front. Comp. Neurosci. 10: 1-17, 2016
%
%   Author: Marko Takanen
%
%   COPYRIGHT (C) 2016 Technical University of Munich
%                      Audio Information Processing
%                      Munich, Germany
%

%% Experimental parameters

fs =1e6; %sample rate (1 MHz)
phaseDur = 40e-6; %duration of the anodic and cathodic phases (in seconds)
ipg = 30e-6;%duration of the inter-phase-gap (in seconds)

levels = (700:10:900)*1e-6; %stimulation levels in amperes 

nTest =500; % number of iterations at each stimulation level

%initiation of the output vectors needed for the plotting
spikingProb = zeros(size(levels));
spikingLatency = zeros(size(levels));

%% Running of the experiments

%Generate a biphasic pulse according to the parameters
pulse = [zeros(100,1);ones(phaseDur*fs,1);zeros(ipg*fs,1);...
    -1*ones(phaseDur*fs,1);zeros(100,1)];

%processing of the pulse at different levels
for levelInd = 1:length(levels)
    I = pulse*levels(levelInd);
    %initiate temporary variables for different iterations at this level
    tempProb = zeros(nTest,1);
    tempLatency = NaN*ones(nTest,1);
    %run the model repeatedly at the given level
    for testInd = 1:nTest
        [tSpk,tempProb(testInd),outStr] = horne2016(I,fs);
        tempLatency(testInd) = outStr.latency;
    end
    %compute the average values, but only from finite values
    % if the model does not spike, values of NaN are returned 
    spikingProb(levelInd) = mean(tempProb(isfinite(tempProb)));
    spikingLatency(levelInd) = mean(tempLatency(isfinite(tempLatency)));
end

%create the figure showing the effect of stimulation level on the (a)
%spiking probability and (b) latency of spiking 
figure;g(1)=subplot(1,2,1);
plot(levels*1e6,spikingProb);
ylabel('Spiking probability');
xlabel('Stimulus level [\muA]');
g(2)=subplot(1,2,2);
plot(levels*1e6,spikingLatency*1e3);
ylabel('Latency of spiking [ms]');
xlabel('Stimulus level [\muA]');
linkaxes(g,'x');
xlim([700 900]);