function [tSpk,Prob_blif,outStruct] = horne2016(insig,fs)
%Horne2016 Compute spiking of electrically stimulated auditory nerve fiber
%
%   Usage: [tSpk,Prob_blif,outStruct] = horne2016(insig,fs)
%
%   Input parameters:
%        insig     : signal used to electrically stimulate the AN fiber. 
%                    Can be either a monophasic, pseudo-monophasic or 
%                    biphasic pulse or a pulse-train sequence
%
%        fs        : sampling rate (by default 1 MHz)
%
%   Output parameters:
%        tSpk      : the time when the AN fiber spikes if it does so.
%                    Otherwise a NaN value is returned
%
%        Prob_blif : the firing probability of the BLIF AN fiber at the
%                    the time of spiking. A value of 0 is returned if the
%                    fiber does not spike. The firing probability can be
%                    used to 
%
%        outStruct : Structure containing additional information about the
%                    time of threshold crossing, latency and jitter, etc.
%
%   Matlab implementation of the Biphasic Leaky Integrate and Fire (BLIF) 
%   model of the electrically simulated auditory nerve fiber by Horne et
%   al. (2016). 
%   
%   The original code by Horne is modified so that all processing takes
%   place in Matlab environment. As a consequence, the resolution for time
%   of firing is limited by the sample rate (1 MHz), unlike the original 
%   code that enabled subsample accuracy by means of interpolation.
%
%   References:
%     C. Horne, C.J. Sumner, and B.U. Seeber. A phenomenological model of
%     the electrically stimulated auditory nerve fiber: temporal and
%     biphasic response properties. Front. Comp. Neurosci. 10: 1-17, 2016
%
%   Implemented by Marko Takanen, TUM, 2016
%
%   COPYRIGHT (C) 2016 Technical University of Munich
%                      Audio Information Processing
%                      Munich, Germany
%*********************************************************************
if(nargin<2)
    fs = 1e6;
else
    insig = resample(insig,1e6,fs);
    fs = 1e6;
end

%% initiate parameters according to the manuscript and the original model
parameters.fs = fs;

% time constant of the fiber
parameters.tau = 2.4841e-04; 

%coeffs for the leaky integrator (1st order IIR) filter 
parameters.bs = [(1-exp(-1/(parameters.tau*fs)))/2 (1-exp(-1/(parameters.tau*fs)))/2];
parameters.as = [1.0000 -exp(-1/(parameters.tau*fs))];

%mean and standard deviation of the threshold of the fiber
parameters.mu = 1.0454e-04;
parameters.sigma = 4.5954e-06;

%minimum duration of the action potential initialization process
parameters.varphi = 3.5e-05;

%parameter that defines the length of the time window within which the
%firing probability is integrated to potentially generate a spike
parameters.omega = 1e-05;

%coefficients required in the computation of the jitter and latency of the
%firing as specified in Table 1 of the manuscript
parameters.jitterCoeffs = [109 3.24 136]*1e-6;
parameters.latencyCoeffs = [106 5.14 368 472]*1e-6;

%% initiate output parameters
tSpk = NaN;
Prob_blif = NaN;

outStruct.tZero = NaN;
outStruct.tOne = NaN;
outStruct.latency = NaN;

%% start the processing 

%leaky integrator to compute the Membrane Voltage
Voltage = filter(parameters.bs,parameters.as,insig);

peakVoltage = cummax(Voltage);

%THR-crossing detector, the time is denoted by tZero

% The thr is randomized separately for each time instant
thr_value = randn(size(insig))*parameters.sigma+parameters.mu;
tZero = find(Voltage>thr_value,1,'first');
if(isempty(tZero))
    return;
end
outStruct.tZero = tZero./parameters.fs;
%find out when the fiber is depolarized by the subsequent phase. 
%This determines the time when the pulse is cancelled 
temp = cumsum(insig(tZero:end));
QZero = tZero+find(temp<0,1,'first');

%check whether the time difference between QZero and tZero is smaller than
%the minimun delay of the action potential initialization process, if it is
%cancel the spike
if (tZero+parameters.varphi*parameters.fs>QZero)
    return
end
%if not, estimate the time tOne, when the initialization process has been
%completed


%the initial estimate is obatained by solving 
%hatTOne(t) = tZero+Y*jit(P(t)) = t, when t e [tZero+phi,QZero]
tempVoltage = peakVoltage(tZero+parameters.varphi*parameters.fs:QZero);
temp = (exprnd(1)*computeJitter_based_on_voltage(tempVoltage,parameters))*ones(1,2)...
        +[-(0:(length(tempVoltage)-1))'./parameters.fs  zeros(size(tempVoltage))];
hatTOne= temp(temp(:,1)<0,2);
if(isempty(hatTOne))
    hatTOne =0;
else
    hatTOne= hatTOne(1);
end
% %once the estimate is derived, the actual time is determined so that it is
% %equal or larger than the minimum duration (varphi)
tOne = round(hatTOne*parameters.fs+parameters.varphi*parameters.fs+tZero);
outStruct.tZero = tOne./parameters.fs;
%check whether the initialization process lasts too long, so that it's
%value exceeds QZero, i.e. the time when the spike is cancelled
if (tOne>QZero)
    return
end
%if not, estimate the time of spiking

%this is done within a time window from tZero-C.omega to tZero+C.omega
%however, there are special limitations that the current cannot be negative
%within that region, otherwise there cannot be any spiking 
timeVect = (tZero-parameters.omega*parameters.fs):(tZero+...
    parameters.omega*parameters.fs);

%again we need to solve the time QZero for each of those time instances
QZeroS = Inf*ones(size(timeVect));
for ind =1:length(timeVect)
    temp = cumsum(insig(timeVect(ind):end));
    temp2= find(temp<0,1,'first');
    if(~isempty(temp2))
         QZeroS(ind) = timeVect(ind)+temp2;
    end
end

%remove time instances that cannot evoke a spike
temp = timeVect+parameters.varphi*parameters.fs-QZeroS;
timeVect = timeVect(temp<0);
QZeroS = QZeroS(temp<0);

if(isempty(timeVect))
    Prob_blif = 0;
else
    %compute  probability that t0 is within the interval [ts(i), ts(i)+dt).
    P = normcdf(peakVoltage(timeVect),parameters.mu,parameters.sigma);
    P = [P(1); diff(P)];
    
    %compute the probability that the spike is not cancelled at different
    %QOs
    tempJitter = computeJitter_based_on_voltage(peakVoltage(min(length(insig)*ones(size(QZeroS)),QZeroS)),parameters)';
    P_exp = 1-exp(-((QZeroS-timeVect)./parameters.fs-parameters.varphi)./...
        tempJitter);
    
    probability = P.*P_exp';
    %integrate over the time window
    Prob_blif = sum(probability);
end
%estimate the time of spiking
tSpk = tZero/parameters.fs+randn*computeJitter_based_on_probability(Prob_blif,parameters)+...
    computeLatency_based_on_probability(Prob_blif,parameters);
outStruct.latency = randn*computeJitter_based_on_probability(Prob_blif,parameters)+...
    computeLatency_based_on_probability(Prob_blif,parameters);
end
%%
%A few additional functions to estimate the jitter and latency as
%described in the Table 1 of Horne et al. (2016)
%
%Note that the "computeJitter_based_on_voltage" and
%"computeJitter_based_on_probability" functions are effectively the same.
%It is just that the former omits the computation of the CDF (cumulative
%distribution function) value and the subsequent computation of the
%quantile function of the normal distribution (probit function) in the
%formula as these cancel each other.

function jitter = computeJitter_based_on_voltage(peakVoltage,param)
    jitter = param.jitterCoeffs(3)./(1+exp(param.jitterCoeffs(2)^-1*(peakVoltage-...
        param.jitterCoeffs(1))));
end

function jitter = computeJitter_based_on_probability(probability,param)
    probit = norminv(probability,param.mu,param.sigma);
    jitter = param.jitterCoeffs(3)./(1+exp(param.jitterCoeffs(2)^-1*(probit-...
        param.jitterCoeffs(1))));
end
function latency = computeLatency_based_on_probability(probability,param)
    probit = norminv(probability,param.mu,param.sigma);
    latency = param.latencyCoeffs(3)./(1+exp(param.latencyCoeffs(2)^-1*(probit-...
        param.latencyCoeffs(1))))+param.latencyCoeffs(4);
end